﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//  NOTE: code inspired by the book "C# Network Programming" by Richard Blum, 
//        published in 2003
//
//*****************************************************************************

using System;
using System.IO;
using System.Net.Sockets;
using TDP.BaseServices.Infrastructure.Serialization;

namespace TDP.BaseServices.Infrastructure.Net
{
    public class SocketHelper
    {
        private SocketHelper()
        {

        }

        private static int SendData(Socket socket, byte[] data)
        {
            int Total = 0;
            int Size = data.Length;
            int Dataleft = Size;
            int Sent;

            while (Total < Size)
            {
                Sent = socket.Send(data, Total, Dataleft, SocketFlags.None);
                Total += Sent;
                Dataleft -= Sent;
            }

            return Total;
        }

        private static byte[] ReceiveData(Socket socket, int size)
        {
            int Total = 0;
            int Recv;
            byte[] Data = new byte[size];
            int Dataleft = size;
            while (Total < size)
            {
                Recv = socket.Receive(Data, Total, Dataleft, 0);
                Total += Recv;
                Dataleft -= Recv;
            }

            return Data;
        }

        private static int SendVarData(Socket socket, byte[] data)
        {
            byte[] DataSize = BitConverter.GetBytes(data.Length);
            int Sent = SendData(socket, DataSize);
            Sent += SendData(socket, data);

            return Sent;
        }

        private static byte[] ReceiveVarData(Socket socket)
        {
            byte[] DataSize = ReceiveData(socket, 4);
            int Size = BitConverter.ToInt32(DataSize, 0);
            byte[] Data = ReceiveData(socket, Size);

            return Data;
        }

        private static byte[] ReceiveData(Stream stream, int size)
        {
            int Total = 0;
            int Recv;
            int Dataleft = size;
            byte[] Data = new byte[size];
            while (Total < size)
            {
                Recv = stream.Read(Data, Total, Dataleft);
                Total += Recv;
                Dataleft -= Recv;
            }

            return Data;
        }

        private static int SendVarData(Stream stream, byte[] data)
        {
            int Size = data.Length;
            byte[] Datasize = new byte[4];
            Datasize = BitConverter.GetBytes(Size);
            stream.Write(Datasize, 0, Datasize.Length);
            stream.Write(data, 0, data.Length);
            return data.Length;
        }

        private static byte[] ReceiveVarData(Stream stream)
        {
            byte[] DataSize = ReceiveData(stream, 4);
            int Size = BitConverter.ToInt32(DataSize, 0);
            byte[] Data = ReceiveData(stream, Size);

            return Data;
        }

        public static void SendObject<T>(Socket socket, T objectToSend)
        {
            byte[] objectSerialized = SerializationHelper.Serialize(objectToSend);
            SendVarData(socket, objectSerialized);
        }

        public static T ReceiveObject<T>(Socket socket)
        {
            byte[] ObjectSerialized = ReceiveVarData(socket);

            if (ObjectSerialized.Length == 0)
                return default(T);
               
            return SerializationHelper.Deserialize<T>(ObjectSerialized);
        }

        public static object ReceiveObject(Socket socket)
        {
            byte[] ObjectSerialized = ReceiveVarData(socket);

            if (ObjectSerialized.Length == 0)
                return null;

            return SerializationHelper.Deserialize(ObjectSerialized);
        }

        public static void SendObject<T>(Stream stream, T objectToSend)
        {
            byte[] objectSerialized = SerializationHelper.Serialize(objectToSend);
            SendVarData(stream, objectSerialized);
        }

        public static T ReceiveObject<T>(Stream stream)
        {
            byte[] ObjectSerialized = ReceiveVarData(stream);

            if (ObjectSerialized.Length == 0)
                return default(T);

            return SerializationHelper.Deserialize<T>(ObjectSerialized);
        }

        public static object ReceiveObject(Stream stream)
        {
            byte[] ObjectSerialized = ReceiveVarData(stream);

            if (ObjectSerialized.Length == 0)
                return null;

            return SerializationHelper.Deserialize(ObjectSerialized);
        }
    }
}
