﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//  Part of this code was taken from:
//  https://docs.microsoft.com/en-us/dotnet/api/system.security.cryptography.rsacryptoserviceprovider?view=netframework-4.7.2
//*****************************************************************************

using System;
using System.Text;
using System.Security.Cryptography;
using TDP.BaseServices.Infrastructure.Logging.Abstract;
using TDP.BaseServices.Infrastructure.Logging;

namespace TDP.BaseServices.Infrastructure.Security
{
    public class AsymmetricCryptography
    {
        private static ILogger _Log = new Logger();

        private AsymmetricCryptography()
        {            
        }

        public static AsymmetricCryptographyKey CreateKey()
        {
            using (var Rsa = new RSACryptoServiceProvider())
            {
                try
                {
                    return new AsymmetricCryptographyKey(Rsa.ExportParameters(true));
                }
                finally
                {
                    Rsa.PersistKeyInCsp = false;
                }
            }
        }

        public static byte[] SignData(byte[] message, AsymmetricCryptographyKey privateKey)
        {
            // The array to store the signed message in bytes
            byte[] SignedBytes = null;
            using (var Rsa = new RSACryptoServiceProvider())
            {            
                try
                {
                    // Import the private key used for signing the message
                    Rsa.ImportParameters(privateKey.ToRSAParameters());

                    // Sign the data, using SHA512 as the hashing algorithm 
                    SignedBytes = Rsa.SignData(message, CryptoConfig.MapNameToOID("SHA512"));
                }
                catch (CryptographicException ex)
                {
                    _Log.Error("AsymmetricCryptography->SignData(string message, AsymmetricCryptographyKey xmlKey) error", ex);
                }
                finally
                {
                    // Set the keycontainer to be cleared when rsa is garbage collected.
                    Rsa.PersistKeyInCsp = false;
                }
            }

            return SignedBytes;
        }

        public static bool VerifyData(byte[] originalMessage, byte[] signedMessage, AsymmetricCryptographyKey publicKey)
        {
            bool Success = false;
            using (var Rsa = new RSACryptoServiceProvider())
            {
                try
                {
                    Rsa.ImportParameters(publicKey.ToRSAParameters());

                    Success = Rsa.VerifyData(originalMessage, CryptoConfig.MapNameToOID("SHA512"), signedMessage);
                }
                catch (CryptographicException e)
                {
                    _Log.Error("AsymmetricCryptography->VerifyData(byte[] originalMessage, byte[] signedMessage, AsymmetricCryptographyKey publicKey) error", e);
                }
                finally
                {
                    Rsa.PersistKeyInCsp = false;
                }
            }
            return Success;
        }

        public static byte[] EncryptData(byte[] data, AsymmetricCryptographyKey publicKey)
        {
            byte[] EncryptedBytes = null;
            using (var Rsa = new RSACryptoServiceProvider())
            {
                try
                {
                    Rsa.ImportParameters(publicKey.ToRSAParameters());
                    EncryptedBytes = Rsa.Encrypt(data, false);
                }
                catch (CryptographicException ex)
                {
                    _Log.Error("AsymmetricCryptography->EncryptData(byte[] data, AsymmetricCryptographyKey publicKey) error", ex);
                }
                finally
                {
                    Rsa.PersistKeyInCsp = false;
                }
            }

            return EncryptedBytes;
        }

        public static byte[] DecrpytData(byte[] data, AsymmetricCryptographyKey privateKey)
        {
            byte[] DecryptedBytes = null;
            using (var Rsa = new RSACryptoServiceProvider())
            {
                try
                {
                    Rsa.ImportParameters(privateKey.ToRSAParameters());

                    DecryptedBytes = Rsa.Encrypt(data, false);
                }
                catch (CryptographicException ex)
                {
                    _Log.Error("AsymmetricCryptography->DecryptData(byte[] data, AsymmetricCryptographyKey privateKey) error", ex);
                }
                finally
                {
                    Rsa.PersistKeyInCsp = false;
                }
            }

            return DecryptedBytes;
        }
    }
}
