﻿//*****************************************************************************
//
//  By The Dummy Programmer
//  https://www.thedummyprogrammer.com
//
//*****************************************************************************

using System;
using System.Text;

namespace TDP.BaseServices.Infrastructure.Security
{
    [Serializable]
    public class AuthenticationToken<T>
    {
        public AuthenticationToken(T clientID, DateTime loginDate, DateTime expireDate, AsymmetricCryptographyKey? key)
        {
            ClientID = clientID;
            LoginDate = loginDate;
            ExpireDate = expireDate;
            
            if (key != null)
                SignedData = AsymmetricCryptography.SignData(MakePayLoad(clientID, loginDate, expireDate), (AsymmetricCryptographyKey)key);
        }

        private byte[] MakePayLoad(T clientID, DateTime loginDate, DateTime expireDate)
        {
            UTF8Encoding Encoder = new UTF8Encoding();
            return Encoder.GetBytes(clientID.ToString() + loginDate.ToString() + expireDate.ToString()); 
        }

        public T ClientID { get; private set; }
        public DateTime LoginDate { get; private set; }
        public DateTime ExpireDate { get; private set; }
        public byte[] SignedData { get; private set; }

        public bool IsSignValid(AsymmetricCryptographyKey key)
        {
            return AsymmetricCryptography.VerifyData(MakePayLoad(ClientID, LoginDate, ExpireDate), SignedData, key);
        }

        public bool IsExpiring(int minutesLeft)
        {
            return (DateTime.Now > ExpireDate.AddMinutes(-minutesLeft));
        }

        public bool IsExpired()
        {
            return IsExpiring(0);
        }

        public bool IsValid(AsymmetricCryptographyKey key, int tokenDuration)
        {
            return IsSignValid(key) && !IsExpired();
        }
    }
}
